#pragma once

#include "sdl.hpp"
#include "transform2d.hpp"
#include "entity.hpp"

#include <iostream>

class Entity
{
protected:
    SDL2::Renderer::SP renderer_;
    Vector2D position_;
    Vector2D scale_;
    float rotation_radians_;

public:
    using SP = std::shared_ptr<Entity>;

    Transform2D transform_;

    Entity(SDL2::Renderer::SP renderer) : renderer_{renderer}, position_{0,0}, scale_{1,1}, rotation_radians_{0} {};
    virtual void update(float delta) = 0;
    virtual void render() = 0;
    virtual void init() {};

    inline decltype(position_) position() const { return position_; }
    inline decltype(scale_) scale() const { return scale_; }
    inline decltype(rotation_radians_) rotation_radians() const { return rotation_radians_; }

    inline void set_position(decltype(position_) const &p)
    {
        position_ = p;
        update_transform();
    }
    inline void set_scale(decltype(scale_) const &s)
    {
        scale_ = s;
        update_transform();
    }
    inline void set_rotation_radians(decltype(rotation_radians_) const &r)
    {
        rotation_radians_ = r;
        update_transform();
    }

    inline virtual void update_transform()
    {
        transform_ = Transform2D::translation(position_) * Transform2D::rotation(rotation_radians_) * Transform2D::scale(scale_.x, scale_.y);
    }
};
