#include "sprite.hpp"

#include <array>
#include <iostream>
#include <algorithm>

Sprite::Sprite(SDL2::Texture::UP texture) : texture_{std::move(texture)}

{
    int width, height;

    SDL_QueryTexture(texture_.get(), nullptr, nullptr, &width, &height);

    corners_ = {
        Vector2D(-width * 0.5f, -height * 0.5f),
        Vector2D(width * 0.5f, -height * 0.5f),
        Vector2D(width * 0.5f, height * 0.5f),
        Vector2D(-width * 0.5f, height * 0.5f)};
}

void Sprite::draw(SDL2::Renderer::SP canvas)
{
    std::array<SDL_Vertex, 4> vertices;

    // Transform the above vectors to SDL vertices
    std::transform(corners_.begin(), corners_.end(),
                   corner_uvs_.begin(), vertices.begin(),

                   [&](auto p, auto uv) -> SDL_Vertex
                   {
                       Vector2D transformed_p = transform * p;
                       return {.position = {transformed_p.x, transformed_p.y},
                               .color = {255, 255, 255, 255},
                               .tex_coord = {uv.x, uv.y}};
                   });

    SDL_RenderGeometry(canvas.get(), texture_.get(), vertices.data(), 4,
                       vertex_indices_.data(), vertex_indices_.size());
}