#pragma once

#include "vector2d.hpp"

class Transform2D
{
public:
    Vector2D u_base, v_base, offset;
    /**
     * @brief Construct a new Transform2D object, equivalent to identity.
     */
    Transform2D() : u_base{1, 0}, v_base{0, 1}, offset{} {}

    inline Vector2D operator*(Vector2D const &v) const
    {
        return offset + Vector2D{u_base.dot(v), v_base.dot(v)};
    }

    inline Transform2D operator*(Transform2D const &other) const
    {
        Transform2D result;
        result.u_base = {Vector2D(u_base.x, v_base.x).dot(other.u_base),
                         Vector2D(u_base.x, v_base.x).dot(other.v_base)};

        result.v_base = {Vector2D(u_base.y, v_base.y).dot(other.u_base),
                         Vector2D(u_base.y, v_base.y).dot(other.v_base)};

        result.offset = {Vector2D(u_base.x, v_base.x).dot(other.offset) + offset.x,
                         Vector2D(u_base.y, v_base.y).dot(other.offset) + offset.y};

        return result;
    }

    static inline Transform2D translation(Vector2D const &offset)
    {
        Transform2D translated;
        translated.offset = offset;

        return translated;
    }

    static inline Transform2D rotation(float angle_r)
    {
        Transform2D rotated;
        rotated.u_base = {std::cos(angle_r), std::sin(angle_r)};
        rotated.v_base = {-std::sin(angle_r), std::cos(angle_r)};

        return rotated;
    }

    static inline Transform2D scale(float x, float y)
    {
        Transform2D scaled;
        scaled.u_base *= x;
        scaled.v_base *= y;

        return scaled;
    }

    static inline Transform2D scale(float scalar)
    {
        return scale(scalar, scalar);
    }

    friend std::ostream& operator<<(std::ostream& os, const Transform2D& t) {
        os << "X: " << t.u_base << " Y: " << t.u_base << " Offset: " << t.u_base << ")";
        return os;
    }
};
