#include "paddle.hpp"

#include "input.hpp"


Paddle::Paddle(SDL2::Renderer::SP renderer)
    : Entity(renderer),
      Collidable(std::make_unique<PolygonShape>(
          std::initializer_list{Vector2D{-0.5, -0.5}, Vector2D{0.5, -0.5}, Vector2D{0.5, 0.5}, Vector2D{-0.5, 0.5}})),
      sprite_(SDL2::Texture::load(renderer, "res/paddle.svg")),
      shape_{static_cast<PolygonShape &>(*Collidable::collision_shape_)}
{
}

void Paddle::render()
{
    sprite_.transform = transform_;
    sprite_.draw(renderer_);

    #ifdef DEBUG_DRAW
    shape_.debug_draw(renderer_);
    aabb_.debug_draw(renderer_);
    #endif
}

void Paddle::update(float delta)
{
    static float current_angle = angle_;
    static Vector2D arm;

    angle_ += rotation_speed_radians_ * (Input::is_key_pressed(SDL_SCANCODE_RIGHT) - Input::is_key_pressed(SDL_SCANCODE_LEFT));

    current_angle = std::lerp(current_angle, angle_, friction_ * delta);
    arm = Transform2D::rotation(current_angle * M_PI * delta) * Vector2D(0,1);

    set_position(pivot_point_ + arm * (300 - 8));
    set_rotation_radians(-arm.angle() + (M_PI / 2.0f));
}